﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Threading;

namespace ClockControlDemo
{
    /// <summary>
    /// Follow steps 1a or 1b and then 2 to use this custom control in a XAML file.
    ///
    /// Step 1a) Using this custom control in a XAML file that exists in the current project.
    /// Add this XmlNamespace attribute to the root element of the markup file where it is 
    /// to be used:
    ///
    ///     xmlns:MyNamespace="clr-namespace:ClockControlDemo"
    ///
    ///
    /// Step 1b) Using this custom control in a XAML file that exists in a different project.
    /// Add this XmlNamespace attribute to the root element of the markup file where it is 
    /// to be used:
    ///
    ///     xmlns:MyNamespace="clr-namespace:ClockControlDemo;assembly=ClockControlDemo"
    ///
    /// You will also need to add a project reference from the project where the XAML file lives
    /// to this project and Rebuild to avoid compilation errors:
    ///
    ///     Right click on the target project in the Solution Explorer and
    ///     "Add Reference"->"Projects"->[Browse to and select this project]
    ///
    ///
    /// Step 2)
    /// Go ahead and use your control in the XAML file.
    ///
    ///     <MyNamespace:ClockControl/>
    ///
    /// </summary>
    public class ClockControl : Control
    {
        static ClockControl()
        {
            DefaultStyleKeyProperty.OverrideMetadata(typeof(ClockControl), new FrameworkPropertyMetadata(typeof(ClockControl)));
        }

        public DateTime? Time
        {
            get { return (DateTime?)GetValue(TimeProperty); }
            set { SetValue(TimeProperty, value); }
        }

        public bool TrackCurrentTime
        {
            get { return (bool)GetValue(TrackCurrentTimeProperty); }
            set { SetValue(TrackCurrentTimeProperty, value); }
        }

        public static readonly DependencyProperty TimeProperty =
            DependencyProperty.Register("Time", typeof(DateTime?), typeof(ClockControl), new UIPropertyMetadata(null));


        public static readonly DependencyProperty TrackCurrentTimeProperty =
            DependencyProperty.Register("TrackCurrentTime", typeof(bool), typeof(ClockControl), new UIPropertyMetadata(false, OnTrackCurrentTimeChanged));

        private static void OnTrackCurrentTimeChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            var control = d as ClockControl;
            if (control == null) return;
            control.OnTrackCurrentTimeChanged();
        }

        private DispatcherTimer _timer;

        private void OnTrackCurrentTimeChanged()
        {
            if (TrackCurrentTime)
            {
                _timer = new DispatcherTimer(TimeSpan.FromMilliseconds(100), DispatcherPriority.Normal, OnTimer,
                                             Dispatcher.CurrentDispatcher);
                _timer.Start();
            }
            else
            {
                if (_timer != null)
                {
                    _timer.Stop();
                }
            }
        }

        private void OnTimer(object sender, EventArgs e)
        {
            Time = DateTime.Now;
        }
    }
}
